'use client';
import { z } from 'zod';
import { zodResolver } from '@hookform/resolvers/zod';
import { useForm } from 'react-hook-form';
import {
  Card,
  CardContent,
  CardDescription,
  CardHeader,
  CardTitle,
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
  Input,
} from '@/components/ui';
import Link from 'next/link';
import { forgotPassword } from '@/lib/queries';
import { useAuth } from '@/lib/hooks/context';
import { useEffect, useState, type ComponentProps } from 'react';
import { useRouter } from 'next/navigation';
import { useSupabaseClient } from '@/utils/supabase';
import { StatusMessage, SubmitButton } from '@/components/default/forms';
import { cn } from '@/lib/utils';

const forgotPasswordFormSchema = z.object({
  email: z.string().email({
    message: 'Please enter a valid email address.'
  }),
});

type ForgotPasswordCardProps = {
  cardProps?: ComponentProps<typeof Card>;
  cardTitleProps?: ComponentProps<typeof CardTitle>;
  cardDescriptionProps?: ComponentProps<typeof CardDescription>;
  signUpLinkProps?: Omit<ComponentProps<typeof Link>, 'href'>;
  formProps?: Omit<ComponentProps<'form'>, 'onSubmit'>;
  formLabelProps?: ComponentProps<typeof FormLabel>;
  buttonProps?: ComponentProps<typeof SubmitButton>;
};

export const ForgotPasswordCard = ({
  cardProps,
  cardTitleProps,
  cardDescriptionProps,
  signUpLinkProps,
  formProps,
  formLabelProps,
  buttonProps = {
    pendingText: 'Sending Reset Link...',
  },
}: ForgotPasswordCardProps) => {
  const router = useRouter();
  const { isAuthenticated, loading, refreshUser } = useAuth();
  const [statusMessage, setStatusMessage] = useState('');
  const supabase = useSupabaseClient();

  const form = useForm<z.infer<typeof forgotPasswordFormSchema>>({
    resolver: zodResolver(forgotPasswordFormSchema),
    defaultValues: {
      email: '',
    },
  });

  useEffect(() => {
    if (isAuthenticated) router.push('/')
  }, [isAuthenticated, router]);

  const handleForgotPassword = async (values: z.infer<typeof forgotPasswordFormSchema>) => {
    try {
      setStatusMessage('');
      const formData = new FormData();
      formData.append('email', values.email);
      if (!supabase) throw new Error('Supabase client not found');
      const result = await forgotPassword(supabase, formData);
      if (result.error) throw new Error(result.error.message);
      await refreshUser();
      setStatusMessage('Check your email for a link to reset your password.');
      form.reset();
      router.push('');
    } catch (error) {
      setStatusMessage(
        `Error: ${error instanceof Error ? error.message : 'Could not sign in!'}`,
      );
    }
  };

  return (
    <Card
      {...cardProps}
      className={cn('min-w-xs sm:min-w-sm bg-card/50', cardProps?.className)}
    >
      <CardHeader>
        <CardTitle
          {...cardTitleProps}
          className={cn('text-2xl font-medium', cardTitleProps?.className)}
        >
          Forgot Password
        </CardTitle>
        <CardDescription
          {...cardDescriptionProps}
          className={cn('text-sm text-foreground', cardDescriptionProps?.className)}
        >
          Don&apos;t have an account?{' '}
          <Link
            {...signUpLinkProps}
            href='/sign-up'
            className={cn('font-medium underline', signUpLinkProps?.className)}
          >
            Sign up!
          </Link>
        </CardDescription>
      </CardHeader>
      <CardContent>
        <Form {...form}>
          <form
            {...formProps}
            onSubmit={form.handleSubmit(handleForgotPassword)}
            className={cn('flex flex-col min-w-64 space-y-6', formProps?.className)}
          >
            <FormField
              control={form.control}
              name='email'
              render={({ field }) => (
                <FormItem>
                  <FormLabel
                    {...formLabelProps}
                    className={cn('text-xl', formLabelProps?.className)}
                  >
                    Email
                  </FormLabel>
                  <FormControl>
                    <Input
                      type='email'
                      placeholder='you@example.com'
                      {...field}
                    />
                  </FormControl>
                  <FormMessage />
                </FormItem>
              )}
            />
            <SubmitButton
              disabled={loading}
              {...buttonProps}
            >
              Reset Password
            </SubmitButton>
            {statusMessage &&
              (statusMessage.includes('Error') ||
              statusMessage.includes('error') ||
              statusMessage.includes('failed') ||
              statusMessage.includes('invalid') ? (
                <StatusMessage message={{ error: statusMessage }} />
              ) : (
                <StatusMessage message={{ success: statusMessage }} />
              ))}
          </form>
        </Form>
      </CardContent>
    </Card>
  );
};
